@@ -1,6 +1,6 @@
 
 ############################################################################
-#    Copyright (C) 2005, 2009 Ritesh Raj Sarraf                            #
+#    Copyright (C) 2005, 2015 Ritesh Raj Sarraf                            #
 #    rrs@researchut.com                                                    #
 #                                                                          #
 #    This program is free software; you can redistribute it and#or modify  #
@@ -23,15 +23,26 @@ import os
 import sys
 
 import threading
-import Queue
 
 import zipfile
 import bz2
 import gzip
 
-# Test Comment
-# For Forest
+import errno
+import shutil
 
+import warnings
+
+
+# LZMA is not native to Python in 2.x
+modLZMA = True
+try:
+        import lzma
+except ImportError:
+        modLZMA = False
+        sys.stderr.write("WARN: lzma module unavailable\n")
+        sys.stderr.write("WARN: Please install python lzma module for APT lzma backend\n")
+                
 WindowColor = True
 try:
         import WConio
@@ -62,98 +73,100 @@ except ImportError:
     
 class Checksum:
         
-        def HashMessageDigestAlgorithms( self, checksum, HashType, file ):
-                data = open( file, 'rb' )
-                if HashType == "sha256":
-                        Hash = self.sha256( data )
-		elif HashType == "md5" or HashType == "md5sum":
-                        Hash = self.md5( data )
-                else: Hash = None
-                data.close()
-                
-                if Hash == checksum:
-                        return True
-                return False
+    def HashMessageDigestAlgorithms( self, checksum, HashType, checksumFile ):
+            
+        try:
+            data = open( checksumFile, 'rb' )
+            if HashType == "sha256":
+                Hash = self.sha256( data )
+            elif HashType == "md5" or HashType == "md5sum":
+                Hash = self.md5( data )
+            else: Hash = None
+        except IOError:
+            return False
+        data.close()
         
-        def sha256( self, data ):
-                hash = hashlib.sha256()
-                hash.update( data.read() )
-                return hash.hexdigest()
+        if Hash == checksum:
+            return True
+        return False
+    
+    def sha256( self, data ):
+        sha256 = hashlib.sha256()
+        sha256.update( data.read() )
+        return sha256.hexdigest()
+    
+    def md5( self, data ):
+        md5hash = hashlib.md5()
+        md5hash.update( data.read() )
+        return md5hash.hexdigest() 
+    
+    def CheckHashDigest( self, checksumFile, checksum ):
+        '''Return Bool against file and its checksum'''
         
-        def md5( self, data ):
-                hash = hashlib.md5()
-                hash.update( data.read() )
-                return hash.hexdigest() 
-        
-        def CheckHashDigest( self, file, checksum ):
-                type = checksum.split(":")[0]
-                type = type.lower()
-                checksum = checksum.split( ":" )[1]
-                return self.HashMessageDigestAlgorithms( checksum, type, file )
+        checksumType = checksum.split(":")[0]
+        checksumType = checksumType.lower()
+        checksum = checksum.split( ":" )[1]
+        return self.HashMessageDigestAlgorithms( checksum, checksumType, checksumFile )
         
 
 class Log:
-        '''A OOP implementation for logging.
-            warnings is to tackle the warning option
-            verbose is to tackle the verbose option
-            color is if you want to colorize your output
-    
-            You should pass these options, taking it from optparse/getopt,
-            during instantiation'''
+        '''
+        To display color on Windows CMD, we have optional dependency on WConio
+        
+        WConio can provide simple coloring mechanism for Microsoft Windows console
+        Color Codes:
+        Black = 0
+        Green = 2
+        Red = 4
+        White = 15
+        Light Red = 12
+        Light Cyan = 11
             
-        #            WConio can provide simple coloring mechanism for Microsoft Windows console
-        #            Color Codes:
-        #            Black = 0
-        #            Green = 2
-        #            Red = 4
-        #            White = 15
-        #            Light Red = 12
-        #            Light Cyan = 11
-        #            
-        #            #FIXME: The Windows Command Interpreter does support colors natively. I think that support has been since Win2k.
-        #            That's all for Windows Command Interpreter.
-        #            
-        #            As for ANSI Compliant Terminals (which most Linux/Unix Terminals are.).....
-        #            I think the ANSI Color Codes would be good enough for my requirements to print colored text on an ANSI compliant terminal.
-        #
-        #            The ANSI Terminal Specification gives programs the ability to change the text color or background color.
-        #            An ansi code begins with the ESC character [^ (ascii 27) followed by a number (or 2 or more separated by a semicolon) and a letter.
-        #    
-        #            In the case of colour codes, the trailing letter is "m"...
-        #    
-        #            So as an example, we have ESC[31m ... this will change the foreground colour to red.
-        #    
-        #            The codes are as follows:
-        #    
-        #            For Foreground Colors
-        #            1m - Hicolour (bold) mode
-        #            4m - Underline (doesn't seem to work)
-        #            5m - BLINK!!
-        #            8m - Hidden (same colour as bg)
-        #            30m - Black
-        #            31m - Red
-        #            32m - Green
-        #            33m - Yellow
-        #            34m - Blue
-        #            35m - Magenta
-        #            36m - Cyan
-        #            37m - White
-        #    
-        #            For Background Colors
-        #    
-        #            40m - Change Background to Black
-        #            41m - Red
-        #            42m - Green
-        #            43m - Yellow
-        #            44m - Blue
-        #            45m - Magenta
-        #            46m - Cyan
-        #            47m - White
-        #    
-        #            7m - Change to Black text on a White bg
-        #            0m - Turn off all attributes.
-        #    
-        #            Now for example, say I wanted blinking, yellow text on a magenta background... I'd type ESC[45;33;5m
+        #FIXME: The Windows Command Interpreter does support colors natively. I think that support has been since Win2k.
+        That's all for Windows Command Interpreter.
+            
+        As for ANSI Compliant Terminals (which most Linux/Unix Terminals are.).....
+        I think the ANSI Color Codes would be good enough for my requirements to print colored text on an ANSI compliant terminal.
+
+        The ANSI Terminal Specification gives programs the ability to change the text color or background color.
+        An ansi code begins with the ESC character [^ (ascii 27) followed by a number (or 2 or more separated by a semicolon) and a letter.
+    
+        In the case of colour codes, the trailing letter is "m"...
+    
+        So as an example, we have ESC[31m ... this will change the foreground colour to red.
+    
+        The codes are as follows:
+    
+        For Foreground Colors
+        1m - Hicolour (bold) mode
+        4m - Underline (doesn't seem to work)
+        5m - BLINK!!
+        8m - Hidden (same colour as bg)
+        30m - Black
+        31m - Red
+        32m - Green
+        33m - Yellow
+        34m - Blue
+        35m - Magenta
+        36m - Cyan
+        37m - White
+    
+        For Background Colors
+    
+        40m - Change Background to Black
+        41m - Red
+        42m - Green
+        43m - Yellow
+        44m - Blue
+        45m - Magenta
+        46m - Cyan
+        47m - White
+    
+        7m - Change to Black text on a White bg
+        0m - Turn off all attributes.
+    
+        Now for example, say I wanted blinking, yellow text on a magenta background... I'd type ESC[45;33;5m
+        '''
         
         def __init__( self, verbose, lock=None ):
                 self.VERBOSE = bool( verbose )
@@ -170,7 +183,7 @@ class Log:
                         self.platform = 'posix'
                         self.color = {'Red': '31m', 'Black': '30m',
                                       'Green': '32m', 'Yellow': '33m',
-                                      'Blue': '34m', 'Magneta': '35m',
+                                      'Blue': '34m', 'Magenta': '35m',
                                       'Cyan': '36m', 'White': '37m',
                                       'Bold_Text': '1m', 'Underline': '4m',
                                       'Blink': '5m', 'SwitchOffAttributes': '0m'}
@@ -198,7 +211,7 @@ class Log:
         def msg( self, msg ):
                 '''Print general messages. If locking is available use them.'''
                 if self.lock:
-                        self.DispLock.acquire( True )
+                        self.DispLock.acquire()
           
                 #self.set_color( 'White' )
                 sys.stdout.write( msg )
@@ -207,11 +220,24 @@ class Log:
         
                 if self.lock:
                         self.DispLock.release()
+
+        def warn( self, msg ):
+                '''Print messages with a warningr. If locking is available use them.'''
+                if self.lock:
+                        self.DispLock.acquire()
+            
+                self.set_color( 'Magenta' )
+                sys.stderr.write( "WARN: " + msg )
+                sys.stderr.flush()
+                self.set_color( 'SwitchOffAttributes' )
+        
+                if self.lock:
+                        self.DispLock.release()
         
         def err( self, msg ):
                 '''Print messages with an error. If locking is available use them.'''
                 if self.lock:
-                        self.DispLock.acquire( True )
+                        self.DispLock.acquire()
             
                 self.set_color( 'Red' )
                 sys.stderr.write( "ERROR: " + msg )
@@ -224,7 +250,7 @@ class Log:
         def success( self, msg ):
                 '''Print messages with a success. If locking is available use them.'''
                 if self.lock:
-                        self.DispLock.acquire( True )
+                        self.DispLock.acquire()
             
                 self.set_color( 'Green' )
                 sys.stdout.write( msg )
@@ -238,7 +264,7 @@ class Log:
         def verbose( self, msg ):
                 '''Print verbose messages. If locking is available use them.'''
                 if self.lock:
-                        self.DispLock.acquire( True )
+                        self.DispLock.acquire()
                 
                 if self.VERBOSE is True:
                         self.set_color( 'Cyan' )
@@ -268,6 +294,8 @@ class ProgressBar( object ):
                 self.min = minValue
                 self.max = maxValue
                 self.span = float( self.max - self.min )
+                if self.span == 0:
+                    self.span = 1
                 self.fd = fd
                 self.signal_set = False
         
@@ -317,7 +345,7 @@ class ProgressBar( object ):
                 self.display()
         
         def display( self ):
-                print "\r%3s /%3s items: %s\r" % ( self.complete, self.items, str( self ) ),
+                sys.stdout.write("\r%3s / %3s items: %s\r" % ( self.complete, self.items, str( self ) ))
         
         def __str__( self ):
                 #compute display fraction
@@ -335,6 +363,14 @@ class ProgressBar( object ):
                 return ( "%d KiB" % ( size ) )
 
 
+class AptOfflineErrors(Exception):
+    def __init__(self, message):
+        Exception.__init__(self, message)
+
+class AptOfflineLibShutilError(Exception):
+    def __init__(self, message):
+        Exception.__init__(self, message)
+        
 class Archiver:
         def __init__( self, lock=None ):
                 if lock is None or lock != 1:
@@ -342,7 +378,12 @@ class Archiver:
                 else:
                         self.ZipLock = threading.Lock()
                         self.lock = True
-        
+
+                #INFO: Needed for bug reports in multiple threads. Because you can have multiple bug reports
+                # for the same src package
+                # https://github.com/rickysarraf/apt-offline/issues/44
+                self.file_possibly_deleted = False
+                
         def TarGzipBZ2_Uncompress( self, SourceFileHandle, TargetFileHandle ):
                 try:
                         TargetFileHandle.write( SourceFileHandle.read() )
@@ -362,7 +403,7 @@ class Archiver:
         
         def compress_the_file( self, zip_file_name, files_to_compress ):
                 '''Condenses all the files into one single file for easy transfer'''
-        
+
                 try:
                         if self.lock:
                                 self.ZipLock.acquire( True )
@@ -374,23 +415,41 @@ class Archiver:
 
                         fileOpened = False
                         try:
-                            filename = zipfile.ZipFile( zip_file_name, "w" )
-                            fileOpened = True
+                                filename = zipfile.ZipFile( zip_file_name, "w" )
+                                fileOpened = True
                         except IOError:
-                            fileOpened = False
+                                fileOpened = False
                 if fileOpened: #Supported from Python 2.5 ??
-                        filename.write( files_to_compress, os.path.basename( files_to_compress ), zipfile.ZIP_DEFLATED )                        
-                        filename.close()
-        
-                        if self.lock:
+
+                        #INFO: We could get duplicate files being writted to the zip archive.
+                        # See explanation below in the exception
+                        warnings.filterwarnings('error')
+
+                        try:
+                                filename.write( files_to_compress, os.path.basename( files_to_compress ), zipfile.ZIP_DEFLATED )
+                        except OSError as e:
+                                if e.errno == errno.ENOENT:
+                                        #INFO: We could be here, because in another thread (amd64), it completed, i.e. wrote to the archive and removed
+                                        # And by the time this thread got a chance, the file was deleted by the previous thread
+                                        #
+                                        # A more ideal fix will be to check for files_to_compress's presence in zipfile at this stage
+                                        self.file_possibly_deleted = True
+                                        raise AptOfflineErrors("Ignoring err ENOENT, multiarch package name clash %s\n" % (files_to_compress))
+                        except UserWarning as e:
+                            raise AptOfflineErrors("Ignoring err type %s\n" % (e.args))
+                        finally:
+                            filename.close()
+                            if self.lock:
                                 self.ZipLock.release()
                         return True
                 else:
+                        if self.lock:
+                                self.ZipLock.release()
                         return False
 
         def decompress_the_file( self, archive_file, target_file, archive_type ):
                 '''Extracts all the files from a single condensed archive file'''
-                if archive_type == "bzip2" or archive_type == "gzip":
+                if archive_type == "bzip2" or archive_type == "gzip" or archive_type == "xz":
                         if archive_type == "bzip2":
                                 try:
                                         read_from = bz2.BZ2File( archive_file, 'r' )
@@ -401,6 +460,14 @@ class Archiver:
                                         read_from = gzip.GzipFile( archive_file, 'r' )
                                 except IOError:
                                         return False
+                        elif archive_type == "xz":
+                                if modLZMA is True:
+                                        try:
+                                                read_from = lzma.LZMAFile(archive_file, 'rb')
+                                        except IOError:
+                                                return False
+                                else:
+                                        return False
                         else:
                                 return False
                                     
@@ -447,12 +514,13 @@ class FileMgmt( object ):
 
         def files( self, root ): 
                 for path, folders, files in os.walk( root ): 
-                        for file in files: 
-                                yield path, file 
+                        for f in files: 
+                                yield path, f 
 
         def find_first_match( self, cache_dir=None, filename=None ):
                 '''Return the full path of the filename if a match is found
                 Else Return False'''
+                
                 if cache_dir is None:
                         return False
                 elif filename is None:
@@ -460,10 +528,10 @@ class FileMgmt( object ):
                 elif os.path.isdir( cache_dir ) is False:
                         return False
                 else:
-                        for path, file in self.files( cache_dir ): 
-                                if file == filename:
-                                        return os.path.join( path, file )
-                                return False
+                        for path, f in self.files( cache_dir ): 
+                                if f == filename:
+                                        return os.path.join( path, f )
+                        return False
         
         def rename_file( self, orig, new ):
                 '''Rename file from orig to new'''
@@ -480,14 +548,41 @@ class FileMgmt( object ):
                         return False
 
         def move_file( self, src, dest ):
-                '''Move file from src to dest.'''
-                if not os.path.isdir( dest ):
-                        return False
-                try:
-                        os.rename( src, dest + "/" + os.path.basename( src ) )
-                except IOError:
-                        return False
+            '''Move file from src to dest.'''
+            if not os.path.isdir( dest ):
+                return False
+            try:
+                shutil.move(src, dest)
+            except IOError:
+                return False
+            except:
+                #INFO: These errors happen when, say, you have duplicate files
+                # shutil didn't show a nice exception handle
+                # So we propagate the failure and warn out loud in the caller
+                raise AptOfflineLibShutilError("Possbile duplicate file %s already present in %s\n" % (src, dest))
+            return True
                 
+        def copy_file(self, src, dest):
+            '''Copy file from src to dest'''
+            try:
+                #INFO: If src and dest are the same, it is effectively opening the same file
+                # in read and write modes, which leads to NULL data corruption
+                destFile = os.path.join(dest, os.path.basename(src))
+                srcFile = os.path.abspath(src)
+                #print(srcFile, destFile)
+                if srcFile == destFile:
+                    return True
+                SFH = open(srcFile, 'rb')
+                DFH = open(destFile, 'wb')
+                
+                DFH.write(SFH.read())
+                DFH.flush()
+                DFH.close()
+                SFH.close()
+            except IOError:
+                SFH.close()
+                return False
+            
         def move_folder( self, src, dest ):
                 '''Move folder from src to dest.'''
                 if os.path.isdir( dest ):
@@ -514,7 +609,7 @@ class FileMgmt( object ):
                                         #self.duplicate_files = set(self.duplicate_files)
                 
                 len = self.duplicate_files.__len__()
-                print len
+                print(len)
                 for x in range( len ):
                         self.duplicate_files[x].sort()
                 self.duplicate_files.sort()
@@ -524,9 +619,9 @@ class FileMgmt( object ):
                         while number < len - 1:
                                 if x in self.duplicate_files[number] or y in self.duplicate_files[number]:
                                         num += 1
-                                        print num
+                                        print(num)
                                         if num > 1:
-                                                print "Num went 2"
+                                                print("Num went 2")
                                                 self.duplicate_files.pop( number )
                                                 num -= 0
                                 number += 1
@@ -551,7 +646,7 @@ class MyThread( threading.Thread ):
                        for i in range( self.threads )
                        ]
                 for thread in self.thread_pool:
-                       thread.guiTerminateSignal=False
+                        thread.guiTerminateSignal=False
         
         def startThreads( self ):
                 for thread in self.thread_pool:
@@ -569,7 +664,7 @@ class MyThread( threading.Thread ):
         def stopQueue( self, timeout=0 ):
                 '''Don't end the program prematurely.
                 (Note that because Queue.get() is blocking by
-                defualt this isn't strictly necessary. But if
+                default this isn't strictly necessary. But if
                 you were, say, handling responses in another
                 thread, you'd want something like this in your
                 main thread.)'''
@@ -581,7 +676,7 @@ class MyThread( threading.Thread ):
                         else:
                                 # let threads also lookout for gui signals of cancellation
                                 thread.join(timeout)
-                                if not thread.isAlive():
+                                if not thread.is_alive():
                                         self.threads_finished += 1
                         
         def run( self, item=None):
